import { resolveDataBinds } from '../common';
import { set as lodashSet } from 'lodash-es';
import { app } from '../../app/global-api';
import { runtime } from '../adapter';
import { untracked } from 'mobx';
import { generateWidgetAPIContext } from '../widgets';

const DEFAULT_MAX_TIMEOUT = 10 * 1000;

export function emitEvent(trigger, listeners = [], args, scopeContext = {}, dataContext, $w, pageId = '') {
  const targetListeners = listeners.filter((l) => l.trigger === trigger);

  async function processListener(listener) {
    // 当前非捕获Event，再判断冒泡行为
    if (!args?.event?.isCapturePhase && listener.noPropagation) {
      args?.domEvent?.stopPropagation && args?.domEvent?.stopPropagation();
    }
    // 判断捕获的执行，只有执行的捕获与配置的捕获一致时。才会执行。
    // 为逻辑分支 额外新增处理

    const event = {
      detail: args.event.detail,
      isCapturePhase: !!args.event?.isCapturePhase,
      origin: args.event,
      name: '',
    };

    if ((listener?.isCapturePhase || false) === (args?.customEventData?.isCapturePhase || false)) {
      if (pageId && $w?.page?.id && pageId !== $w?.page?.id) {
        console.error(`Action error: [${listener.trigger}] 页面生命周期结束，链式调用中断`);
        return;
      }
      if (listener?.sourceKey === 'platform:utils.If') {
        try {
          const res = await invokeListener(listener, args, scopeContext, dataContext, $w);
          event.name = `${listener.key}.${res ? 'success' : 'fail'}`;
        } catch (e) {
          event.name = `${listener.key}.fail`;
        }
      } else {
        try {
          const res = await invokeListener(listener, args, scopeContext, dataContext, $w);
          event.name = `${listener.key}.success`;
          event.detail = res;
        } catch (e) {
          if (e.message) {
            e.message = `${args?.event?.currentTarget?.id || ''}[${listener.trigger}] ${e.message}`;
          }

          event.name = `${listener.key}.fail`;
          event.detail = e;

          // 之前 invoke 内部catch 了错误，不会抛错
          // throw e
          console.error(`Action error: [${listener.trigger}]`, e);
        }
      }
    }

    emitEvent(
      event.name,
      listeners,
      {
        ...args,
        event,
        customEventData: event,
      },
      scopeContext,
      dataContext,
      $w,
      pageId,
    );
  }

  if (!targetListeners.length && typeof trigger === 'string' && /\.fail$/.test(trigger)) {
    app.showToast({
      title: `事件响应失败：${args.event.detail}`,
      icon: 'error',
    });
  } else {
    targetListeners.forEach((listener) => {
      processListener(listener);
    });
  }
}

export function generateEmit(
  $w,
  currentWidget = undefined, // page 节点暂未生成
  injectContext = undefined,
  extraListeners = [],
  extraForContext = {},
) {
  return (trigger, listeners = [], eventData, forItems, domEvent, scopeContext, dataContext, pageId = '') => {
    listeners = listeners.concat(extraListeners);

    if (injectContext?.emit) {
      return injectContext?.emit(trigger, listeners, eventData, forItems, domEvent, scopeContext, dataContext);
    } else {
      const event = {
        detail: eventData,
        name: trigger,
        target: currentWidget,
        currentTarget: currentWidget,
        domEvent,
      };
      const forContext = {
        ...forItems,
        ...extraForContext,
      };
      emitEvent(
        trigger,
        listeners,
        {
          event,
          customEventData: event,
          forItems: forContext,
          domEvent,
        },
        scopeContext,
        dataContext,
        untracked(() => {
          return generateWidgetAPIContext($w, currentWidget, forContext, scopeContext);
        }),
        pageId,
      );
    }
  };
}

function invokeListener(listener, args, scopeContext, dataContext, $w) {
  const { key, instanceFunction, data = {}, dataBinds = {} } = listener;
  if (!instanceFunction) {
    try {
      // @ts-ignore
      window._aegis?.error?.('SYSTEM_ERROR: listener not found', listener);
      console.error('$page current hander', runtime?.['$page']?.handler);
    } catch (e) {}
    throw new Error(`${key} handler 未找到'`);
  }
  // ToDo resolve databinds
  const action = instanceFunction;
  const resolvedData = {
    ...data,
  };
  const resolvedDataBinds = resolveDataBinds(
    dataBinds,
    args.forItems,
    { event: args.event },
    scopeContext,
    true,
    dataContext,
    $w,
  );

  // eslint-disable-next-line no-restricted-syntax
  for (const key in resolvedDataBinds) {
    if (resolvedDataBinds[key] && resolvedDataBinds[key].__type === 'scopedValue') {
      try {
        lodashSet(resolvedData, key, resolvedDataBinds[key].getValue(scopeContext));
      } catch (e) {
        lodashSet(resolvedData, key, '');
      }
    } else {
      lodashSet(resolvedData, key, resolvedDataBinds[key]);
    }
  }
  const params = {
    data: resolvedData,
    ...args,
  };

  return action(params);
}
